/*****************************************************************************
 * Headers / Using Declarations
 *****************************************************************************/

#include <sstream>

#include "clientthreadhandler.h"
#include "mutexmanager.h"
#include "output.h"

using std::ostringstream;

/*****************************************************************************
 * Static member initialization (number of current, live threads)
 *****************************************************************************/

int ClientThreadHandler::mLive = 0;

/*****************************************************************************
 * Class Definition: ClientThreadHandler
 *****************************************************************************/

ClientThreadHandler::ClientThreadHandler(
   int number,
   int clientHandle,
   HANDLE mutex,
   Output *output,
   int id
) :
   mNumber(number),
   mClientHandle(clientHandle),
   mMutex(mutex),
   mOutput(output),
   mID(id)
{
   stateChange(true);
}

ClientThreadHandler::~ClientThreadHandler()
{
   stateChange(false);
}

void ClientThreadHandler::stateChange(bool isBegin)
{
   string errMsg = "";

   if(!MutexManager::acquire(mMutex, errMsg))
   {
      mOutput->write(errMsg);
   }
   else
   {
      if(isBegin)
         ++mLive;
      else
         --mLive;

      int value = mLive;

      if(!MutexManager::release(mMutex, errMsg))
      {
         mOutput->write(errMsg);
      }
      else
      {
         string type = (isBegin ? "Begin" : "End");

         ostringstream oss;

         oss << "[clientHandle: " << mClientHandle << ", "
             << "number: "        << mNumber       << ", "
             << "type: "          << type          << ", "
             << "active: "        << value         << "]";

         mOutput->writeVerbose(_OFLI(mID, oss.str()));
      }
   }
}
