/*****************************************************************************
 * C/C++ Headers
 *****************************************************************************/

#include <stdio.h>
#include <sstream>

using std::ostringstream;

/*****************************************************************************
 * Local Headers
 *****************************************************************************/

#include "cfg.h"
#include "dir.h"
#include "util.h"
#include "config.h"

#define CONFIG_FIELD_SIZE (12)

const string Config::PROGRAM_NAME    = "zproxy";
const string Config::VERSION         = "1.0";
const double Config::DEFAULT_TIMEOUT = 10.0;
const int    Config::DEFAULT_PORT    = 8888;

/*****************************************************************************
 * Class: Config
 *****************************************************************************/

Config::Config()
{
   // Nothing to do ...
}

/*****************************************************************************
 * Utility Functions
 *****************************************************************************/

string Config::toString(string title)
{
   string s = "";

   s += "------------------------------------------------\n";
   s += "Config - " + title + " - Dump\n";
   s += "------------------------------------------------\n";
   s += "\n";

   s += "[settings]\n";

   s += kvToString(CONFIG_PORT,    mPort)        + "\n";
   s += kvToString(CONFIG_DIR,     mDir)         + "\n";
   s += kvToString(CONFIG_RLOG,    mRLog)        + "\n";
   s += kvToString(CONFIG_VERBOSE, mVerbose)     + "\n";
   s += kvToString(CONFIG_TIMEOUT, mTimeout)     + "\n";
   s += kvToString(CONFIG_CONSOLE, mConsole)     + "\n";
   s += kvToString(CONFIG_LOG,     mLog)         + "\n";

   return(s);
}

void Config::dump(string title)
{
   printf("%s\n", toString(title).c_str());
}

string Config::kvToString(string key, string value)
{
   string s =
      "   " + Util::rpad(key, CONFIG_FIELD_SIZE);

   s += " = " + (value == "" ? "[Not Set]" : "'" + value + "'");

   return(s);
}

void Config::debug(string msg)
{
   if(0)
      printf("[DEBUG] %s\n", msg.c_str());
}

void Config::debug(string section, string key, string value)
{
   string s =
      "Section: [" + section + "], " +
      "Key: [" + key + "], Value: [" + value + "]";

   Config::debug(s);
}

/*****************************************************************************
 * Callback Functions
 *****************************************************************************/

int Config::settingsBegin(
   void *voidPtr, string key, string value, string &errMsg
)
{
   debug("Begin: 'settings'");
   return(1);
}

int Config::settingsPort(
   void *voidPtr, string key, string value, string &errMsg
)
{
   debug(CONFIG_SETTINGS, key, value);

   Config *c = (Config *)voidPtr;

   return(Cfg::doInteger(key, value, c->mPort, errMsg));
}

int Config::settingsDir(
   void *voidPtr, string key, string value, string &errMsg
)
{
   debug(CONFIG_SETTINGS, key, value);

   Config *c = (Config *)voidPtr;

   if(!Dir::isRelativePath(value))
   {
      ostringstream oss;

      oss <<
         "Invalid value: [" << value << "] for: [" << key << "] " <<
         "(Relative path expected)";

      errMsg = oss.str();

      return(0);
   }

   c->mDir = value;

   return(1);
}

int Config::settingsRLog(
   void *voidPtr, string key, string value, string &errMsg
)
{
   debug(CONFIG_SETTINGS, key, value);

   Config *c = (Config *)voidPtr;

   return(Cfg::doBoolean(key, value, c->mRLog, errMsg));
}

int Config::settingsVerbose(
   void *voidPtr, string key, string value, string &errMsg
)
{
   debug(CONFIG_SETTINGS, key, value);

   Config *c = (Config *)voidPtr;

   return(Cfg::doBoolean(key, value, c->mVerbose, errMsg));
}

int Config::settingsTimeout(
   void *voidPtr, string key, string value, string &errMsg
)
{
   debug(CONFIG_SETTINGS, key, value);

   Config *c = (Config *)voidPtr;

   return(Cfg::doReal(key, value, c->mTimeout, errMsg));
}

int Config::settingsConsole(
   void *voidPtr, string key, string value, string &errMsg
)
{
   debug(CONFIG_SETTINGS, key, value);

   Config *c = (Config *)voidPtr;

   return(Cfg::doBoolean(key, value, c->mConsole, errMsg));
}

int Config::settingsLog(
   void *voidPtr, string key, string value, string &errMsg
)
{
   debug(CONFIG_SETTINGS, key, value);

   Config *c = (Config *)voidPtr;

   return(Cfg::doBoolean(key, value, c->mLog, errMsg));
}
