#ifndef _CONFIG_H_
#   define _CONFIG_H_

#include <string>

using std::string;

// -- Section Name

#define CONFIG_SETTINGS "settings"

// -- Key Names

#define CONFIG_PORT    "port"
#define CONFIG_DIR     "dir"
#define CONFIG_RLOG    "rlog"
#define CONFIG_VERBOSE "verbose"
#define CONFIG_TIMEOUT "timeout"
#define CONFIG_CONSOLE "console"
#define CONFIG_LOG     "log"

/*****************************************************************************
 * Declaration: struct Config
 *****************************************************************************/

struct Config
{
   static const string PROGRAM_NAME;
   static const string VERSION;
   static const double DEFAULT_TIMEOUT;
   static const int    DEFAULT_PORT;

   // ---------------------------------
   // Our settings ...
   // ---------------------------------

   string mPort;
   string mDir;
   string mRLog;
   string mVerbose;
   string mTimeout;
   string mConsole;
   string mLog;

   // ------------------
   // Constructor
   // ------------------

   Config();

   // ---------------------------------
   // Utility functions
   // ---------------------------------

   string toString(string title);
   void dump(string title);

   static string kvToString(string key, string value);

   static void debug(string msg);
   static void debug(string section, string key, string value);

   // ---------------------------------------------------------
   // Called when starting to process 'settings'
   // ---------------------------------------------------------

   static int settingsBegin(
      void *voidPtr, string key, string value, string &errMsg
   );

   // ---------------------------------------------------------
   // Settings value handler functions
   // ---------------------------------------------------------

   static int settingsPort(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int settingsDir(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int settingsRLog(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int settingsVerbose(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int settingsTimeout(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int settingsConsole(
      void *voidPtr, string key, string value, string &errMsg
   );

   static int settingsLog(
      void *voidPtr, string key, string value, string &errMsg
   );
};

#endif
