#ifndef _DIR_H_
#   define _DIR_H_

#include <string>
#include <vector>
#include <time.h>

using std::string;
using std::vector;

struct DirEntry
{
   string mName;
   string mFullPath;
   time_t mModTime;
   int mIsDirectory;
   int mIsHidden;
   int mIsSystem;
   _fsize_t mSize;

   DirEntry();

   DirEntry(
      string fullPath,
      time_t modTime,
      int isDirectory,
      int isHidden,
      int isSystem,
      _fsize_t size
   );

   void dump();
};

typedef vector<DirEntry> DirEntryVector;

class Dir
{
   public:
      static string sep();
      static char   csep();

      static int hasDriveLetter(string s);
      static int driveExists(char letter);
      static int isAbsolutePath(string s);
      static int isRelativePath(string s);

      static int cwd(string &value, string &errMsg);
      static int cd(string theDir, string &errMsg);

      static int getProgramPath(string &thePath, string &errMsg);

      static int getEntry(string thePath, DirEntry &entry, string &errMsg);

      static int isDir(string theDir);
      static int isFile(string theFile);

      static string toStdForm(string path);
      static int ensureDir(string theDir, string &errMsg);

      static int list(
         string theDirectory, DirEntryVector &theList, string &errMsg
      );

      static int listRegex(
         string regex,
         string theDirectory,
         DirEntryVector &theList,
         string &errMsg
      );

   private:
      static int doList(
         string thePath,
         int doAll,
         DirEntryVector &theList,
         string &errMsg
      );
};

#endif
