/*****************************************************************************
 * Headers / Using Declarations
 *****************************************************************************/

#include <sstream>

#include "logger.h"
#include "util.h"

using std::ostringstream;

/*****************************************************************************
 * Class Definition: Logger
 *****************************************************************************/

Logger::Logger(const string &dir, const string &type, int number)
{
   ostringstream name;

   name << type << "_" << number << ".log";

   string theDir = dir;

   if(!Util::endsWith(dir, "\\"))
      theDir += "\\";

   mFileName = theDir + name.str();
}

bool Logger::write(char *buffer, int n, string &errMsg)
{
   FILE *f = NULL;

   if((f = fopen(mFileName.c_str(), "ab")) == NULL)
   {
      fprintf(
         stderr,
         "Could not open: [%s], Msg: [%s]\n",
         mFileName.c_str(), errMsg.c_str()
      );
   }
   else
   {
      for(int i = 0; i < n; i ++)
      {
         if(fputc(buffer[i], f) == EOF)
         {
            errMsg  = "Error writing to log file: [";
            errMsg += mFileName;
            errMsg += "]";

            if(false)
            {
               extern int errno;

               printf("errno: %d\n", errno);

               if(errno)
               {
                  errMsg += " (";
                  errMsg += strerror(errno);
                  errMsg += ")";
               }
            }

            fclose(f);
            return(false);
         }
      }

      fclose(f);
   }

   return(true);
}

/*****************************************************************************
 * Class Definition: RequestLogger
 *****************************************************************************/

RequestLogger::RequestLogger(const string &dir, int number) :
   Logger(dir, "request", number)
{
}

/*****************************************************************************
 * Class Definition: ResponseLogger
 *****************************************************************************/

ResponseLogger::ResponseLogger(const string &dir, int number) :
   Logger(dir, "response", number)
{
}
