#include <sstream>
#include "mutex.h"

using std::ostringstream;

bool Mutex::create(HANDLE *hPtr, string &errMsg)
{
   HANDLE h = CreateMutex(NULL, FALSE, NULL);

   if(!h)
   {
      ostringstream oss;

      oss << "(" << __FILE__ << "|" << __LINE__ << ") " <<
         "Call to 'CreateMutex' failed";

      errMsg = oss.str();

      return(false);
   }

   *hPtr = h;

   return(true);
}

Mutex::Mutex(HANDLE h) : mH(h)
{
}

Mutex::~Mutex()
{
   CloseHandle(mH);
}
      
bool Mutex::doAction(MutexAction &theAction, string &errMsg)
{
   if(!acquire(errMsg))
      return(false);

   bool returnValue     = theAction.execute(errMsg);
   string releaseErrMsg = "";

   if(!release(releaseErrMsg))
   {
      if(returnValue)
      {
         returnValue = false;
         errMsg      = releaseErrMsg;
      }
      else
      {
         errMsg = errMsg + ", " + releaseErrMsg;
      }
   }

   return(returnValue);
}


bool Mutex::doAction(
   GenericFunction theFunction, void *voidPtr, string &errMsg
)
{
   GenericFunctionMutexAction theAction(voidPtr, theFunction);

   return(doAction(theAction, errMsg));
}

bool Mutex::acquire(string &errMsg)
{
   int status = WaitForSingleObject(mH, INFINITE);

   if(status != 0)
   {
      ostringstream oss;

      oss << "(" << __FILE__ << "|" << __LINE__ << ") " <<
         "Call to 'WaitForSingleObject' (mutex) failed " <<
         "[" << GetCurrentThreadId() << "]";

      errMsg = oss.str();

      return(false);
   }

   return(true);
}

bool Mutex::release(string &errMsg)
{
   if(!ReleaseMutex(mH))
   {
      ostringstream oss;

      oss << "(" << __FILE__ << "|" << __LINE__ << ") " <<
         "Call to 'ReleaseMutex failed";

      errMsg = oss.str();

      return(false);
   }

   return(true);
}
