#ifndef _MUTEX_H_
#   define _MUTEX_H_

#include <windows.h>
#include <string>

using std::string;

typedef bool (*GenericFunction)(void *voidPtr, string &errMsg);

class MutexAction
{
   public:
      virtual ~MutexAction() {}
      virtual bool execute(string &errMsg) { return(true); }
};

class GenericFunctionMutexAction : public MutexAction
{
   public:
      GenericFunctionMutexAction(
         void *voidPtr, GenericFunction theFunction
      ) :
         mVoidPtr(voidPtr), mFunction(theFunction)
      {
      }

      bool execute(string &errMsg)
      {
         return(mFunction(mVoidPtr, errMsg));
      }

   private:
      void            *mVoidPtr;
      GenericFunction mFunction;
};

class Mutex
{
   public:
      Mutex(HANDLE h);
      ~Mutex();

      static bool create(HANDLE *hPtr, string &errMsg);

      bool doAction(MutexAction &theAction, string &errMsg);

      bool doAction(
         GenericFunction theFunction,
         void *voidPtr,
         string &errMsg
      );

   private:
      bool acquire(string &errMsg);
      bool release(string &errMsg);

      HANDLE mH;
};

#endif
