/*****************************************************************************
 * Headers
 *****************************************************************************/

#include "mutexmanager.h"
#include "util.h"

/*****************************************************************************
 * Class Definition: MutexManager
 *****************************************************************************/

MutexManager::MutexManager() : mThread(0), mScreen(0), mLog(0)
{
}

MutexManager::~MutexManager()
{
   HANDLE handles[] = { mThread, mScreen, mLog };
   int n            = USZ(handles);

   for(int i = 0; i < n; i ++)
   {
      if(handles[i])
         CloseHandle(handles[i]);
   }
}

bool MutexManager::setHandles(string &errMsg)
{
   struct Entry
   {
      const char *description;
      const char *name;
      HANDLE *handlePtr;
   };

   Entry entries[] =
   {
      { "active thread count", "threadMutex", &mThread },
      { "screen output",       "screenMutex", &mScreen },
      { "log output",          "logMutex",    &mLog    }
   };

   int n = sizeof(entries) / sizeof(entries[0]);

   for(int i = 0; i < n; i ++)
   {
      HANDLE h = CreateMutex(NULL, FALSE, entries[i].name);

      if(!h)
      {
         errMsg =
            "Call to 'CreateMutex' failed (";

         errMsg += entries[i].description;
         errMsg += ")";

         return(false);
      }

      *(entries[i].handlePtr) = h;
   }

   return(true);
}

HANDLE MutexManager::thread() { return(mThread); }
HANDLE MutexManager::screen() { return(mScreen); }
HANDLE MutexManager::log()    { return(mLog);    }

bool MutexManager::acquire(HANDLE theHandle, string &errMsg)
{
   int status = WaitForSingleObject(theHandle, INFINITE);

   if(status != 0)
   {
      errMsg = "Call to WaitForSingleObject (mutex) failed";
      return(false);
   }

   return(true);
}

bool MutexManager::release(HANDLE theHandle, string &errMsg)
{
   if(!ReleaseMutex(theHandle))
   {
      errMsg = "Call to ReleaseMutex failed";
      return(false);
   }

   return(true);
}
