#ifndef _OUTPUT_H_
#   define _OUTPUT_H_

/*****************************************************************************
 * Headers / Using Declarations
 *****************************************************************************/

#include <windows.h>
#include <string>

using std::string;

#define _OFL(msg)      Output::info(__FILE__, __LINE__, msg)
#define _OFLI(id, msg) Output::info(__FILE__, __LINE__, id, msg)

/*****************************************************************************
 * Class Declaration: Output
 *****************************************************************************/

class Output
{
   public:

      Output(bool doVerbose);
      Output();

      virtual ~Output();
      virtual void write(const string &s);
      virtual void writeVerbose(const string &s);

      void exit(const string &s);

      static string info(const char *file, int line);
      static string info(const char *file, int line, string msg);
      static string info(const char *file, int line, int id, string msg);

   private:

      bool mVerbose;
};

/*****************************************************************************
 * Class Declaration: MutexOutput
 *****************************************************************************/

class Mutex;

class MutexOutput : public Output
{
   public:

      MutexOutput(HANDLE h, bool doVerbose);
      void write(const string &s);

   protected:

      virtual void writeSynchronized(const string &s) = 0;

   private:

      HANDLE mH;
};

/*****************************************************************************
 * Class Declaration: ScreenOutput
 *****************************************************************************/

class ScreenOutput : public MutexOutput
{
   public:
      ScreenOutput(HANDLE mutex, bool doVerbose);
      void writeSynchronized(const string &s);
};

/*****************************************************************************
 * Class Declaration: LogOutput
 *****************************************************************************/

class LogOutput : public MutexOutput
{
   public:

      LogOutput(const string &fileName, HANDLE mutex, bool doVerbose);
      void writeSynchronized(const string &s);

   private:
      string mName;
};

/*****************************************************************************
 * Class Declaration: TeeOutput
 *****************************************************************************/

class TeeOutput : public Output
{
   public:

      TeeOutput(Output *a, Output *b);
      ~TeeOutput();

      void writeVerbose(const string &s);
      void write(const string &s);

   private:

      Output *mA;
      Output *mB;
};

#endif
