#include <stdio.h>
#include "regex.h"

static void doMatch(const char *regex, bool ignoreCase, const char *target)
{
   printf(
      "Regex     : [%s]\n"
      "IgnoreCase: [%s]\n"
      "Target    : [%s]\n",
      regex, ignoreCase ? "Yes" : "No", target
   );

   EntryVector ev;
   string      errMsg;

   if(!Regex::match(regex, ignoreCase, target, ev, errMsg))
   {
      if(errMsg != "")
      {
         printf("%s\n", errMsg.c_str());
      }
      else
      {
         printf("No Match Found\n");
      }
   }
   else
   {
      int n = ev.size();

      printf("%d Match%s Found\n", n, n == 1 ? "" : "es");

      for(int i = 0; i < n; i ++)
      {
         printf("Entry #%d {\n", i + 1);

         for(int j = 0, nn = ev[i].mStrings.size(); j < nn; j ++)
         {
            printf(
               "   { %d, '%s' }\n",
               ev[i].mOffsets[j], ev[i].mStrings[j].c_str()
            );
         }

         printf("}\n");
      }
   }
}

int main()
{
   struct Entry
   {
      const char *regex;
      bool ignoreCase;
      const char *target;
   };

   struct Entry entries[] =
   {
      {
         "DOG",
         false,
         "A string with 'dog' in it"
      },
      {
         "DOG",
         true,
         "A string with 'dog' in it"
      },
      {
         "dog",
         false,
         "A DOG a dog, a doggone good time ..."
      },
      {
         "(\\d{3})-(\\d{2})-(\\d{4})",
         false,
         "882-29-9921 also 585-65-1234 and 222-33-4444"
      },
      {
         "(\\$[a-z_][a-z_0-9]*)",
         true,
         "The variables $x, $y, $z and $bytesPerSecond"
      },
      {
         "^\\s*$",
         true,
         "\t\t   \t\t"
      },
      {
         "\\s*",
         false,
         "\t\t   \t\t"
      }
   };

   int n = sizeof(entries) / sizeof(entries[0]);

   for(int i = 0; i < n; i ++)
   {
      if(i > 0)
         printf("\n");

      doMatch(entries[i].regex, entries[i].ignoreCase, entries[i].target);
   }

   const char *r1 = "[dc][ao][gt]";
   const char *s1 = "A dog a cat and a parrot";
   const char *s2 = "Just the parrot";

   printf("%s => %s => %s\n", r1, s1, Regex::match(r1, s1) ? "Yes" : "No");
   printf("%s => %s => %s\n", r1, s2, Regex::match(r1, s2) ? "Yes" : "No");

   return(0);
}
