#include <stdio.h>
#include <sstream>

using std::ostringstream;

#include "regex.h"
#include "pcre.h"

bool Regex::match(
   const char  *pattern,
   bool        ignoreCase,
   const char  *target,
   EntryVector &instances,
   string      &errMsg
)
{
   int errorOffset      = 0;
   const char *errorPtr = 0;
   pcre *re             = NULL;

   if(0)
      printf("pattern: [%s]\n", pattern);

   errMsg = "";

   re = pcre_compile(
      pattern,
      ignoreCase ? PCRE_CASELESS : 0,
      &errorPtr,
      &errorOffset,
      NULL
   );

   if(!re)
   {
      ostringstream oss;

      oss << "Error: " << errorPtr << ", " << pattern;

      if(pattern + errorOffset < pattern + strlen(pattern))
      {
         // We've NOT been given an errorOffset that lands
         // us at the end of the 'pattern'.
         //
         // Now, we're either at the beginning of
         // the string, or, we'll supply offset information

         if(errorOffset > 0)
         {
            oss << ", at offset: " << errorOffset << ", " <<
               pattern + errorOffset;
         }
      }

      errMsg = oss.str();

      return(false);
   }

   // The pattern was ok.  From here on out,
   // we're obilgated to free(re) before
   // we return.

   int ovector[255] = { 0 };
   int size         = sizeof(ovector) / sizeof(ovector[0]);
   int theOffset    = 0;
   bool returnValue = true;

   while(1)
   {
      const char *segment = target + theOffset;

      if(!*segment)
         break;

      if(0)
      {
         printf(
            "offset: %d, searching segment: '%s'\n", theOffset, segment
         );
      }

      int rv = pcre_exec(
         re,
         NULL,
         segment,
         strlen(segment),
         0,
         0,
         ovector,
         size
      );

      if(0)
         printf("returnValue: [%d]\n", rv);

      if(rv <= 0)
      {
         returnValue = false;

         // The only "error" that doesn't cause us
         // to populate errMsg is a non-match

         if(rv != PCRE_ERROR_NOMATCH)
         {
            ostringstream oss;

            oss << "Error calling pcre_exec, returnCode: [" << rv << "]";

            errMsg = oss.str();
         }
         else
         {
            // A non match, if this is our first
            // pass through the loop (no previous matches)
            // then we return 0 indicating that a match
            // was not found.

            if(theOffset > 0)
               returnValue = true;
         }

         break;
      }

      // Record the matches (strings/offsets)

      Entry theEntry;

      for(int i = 0, j = 0; i < rv; i++, j += 2)
      {
         int start = ovector[j];
         int end   = ovector[j + 1];

         // Work around a bit of weirdness we saw
         // when using: "^\\s*"

         if(start == 0 && end == 0)
         {
            if(rv == 1)
            {
               end = strlen(segment);
            }
         }

         if(0)
         {
            printf("j: %d, start: %d, end: %d\n", j, start, end);
         }

         int n   = end - start;
         char *c = new char[n + 1];

         sprintf(c, "%.*s", n, segment + start);

         string theString = c;

         delete [] c;

         theEntry.mStrings.push_back(theString);

         theEntry.mOffsets.push_back(theOffset + start);

         if(i + 1 == rv)
            theOffset += end;
      }

      if(0)
      {
         for(int k = 0, n = theEntry.mStrings.size(); k < n; k ++)
         {
            printf(
               "   Entry[%d] : { %s, %d }\n",
               k, theEntry.mStrings[k].c_str(), theEntry.mOffsets[k]
            );
         }
      }

      instances.push_back(theEntry);
   }

   free(re);

   if(0)
   {
      int n = instances.size();

      printf("nInstances: %d\n", n);

      for(int k = 0; k < n; k ++)
      {
         Entry theEntry = instances[k];

         for(int m = 0, nn = theEntry.mStrings.size(); m < nn; m ++)
         {
            printf(
               "   Entry[%d] : { %s, %d }\n",
               m, theEntry.mStrings[m].c_str(), theEntry.mOffsets[m]
            );
         }
      }
   }

   return(returnValue);
}

bool Regex::match(
   const char *pattern,
   bool       ignoreCase,
   const char *target,
   string     &errMsg
)
{
   EntryVector ev;

   return(
      Regex::match(pattern, ignoreCase, target, ev, errMsg)
   );
}

bool Regex::match(
   const char *pattern,
   bool       ignoreCase,
   const char *target
)
{
   string errMsg;

   if(!Regex::match(pattern, ignoreCase, target, errMsg))
      return(false);

   return(true);
}

bool Regex::match(
   const char *pattern,
   const char *target,
   string      &errMsg
)
{
   return(Regex::match(pattern, false, target, errMsg));
}

bool Regex::match(
   const char *pattern,
   const char *target
)
{
   return(Regex::match(pattern, false, target));
}
