#ifndef _REGEX_H_
#define _REGEX_H_

#include <vector>
#include <string>

using std::vector;
using std::string;

struct Entry
{
   vector<string> mStrings;
   vector<int>    mOffsets;
};

typedef vector<Entry> EntryVector;

class Regex
{
   public:
      // -----------------------------------
      // Returns:
      //    - true  : a match was found
      //    - false : other
      //
      // In event of an actual error, errMsg
      // will contain a description of
      // the error.  Normally (returning
      // false because of a non-match)
      // errMsg will be empty.
      // -----------------------------------

      static bool match(
         const char  *pattern,
         bool        ignoreCase,
         const char  *target,
         EntryVector &instances,
         string      &errMsg
      );

      static bool match(
         const char  *pattern,
         bool        ignoreCase,
         const char  *target,
         string      &errMsg
      );

      static bool match(
         const char *pattern,
         bool       ignoreCase,
         const char *target
      );

      static bool match(
         const char *pattern,
         const char *target,
         string      &errMsg
      );

      static bool match(
         const char *pattern,
         const char *target
      );
      
   private:
      Regex();
      Regex(const Regex &);
      Regex &operator=(const Regex &);
};
#endif
